"""Runner that outputs the residuals of the linear solver with Gnuplot"""

from StepAnalyzedCommon import StepAnalyzedCommon
from BasicRunner import BasicRunner
from BasicWatcher import BasicWatcher

from PyFoam.LogAnalysis.BoundingLogAnalyzer import BoundingPlotLogAnalyzer
from PyFoam.Basics.GnuplotTimelines import GnuplotTimelines
from PyFoam.Basics.TimeLineCollection import signedMax

class GnuplotCommon(StepAnalyzedCommon):
    """Class that collects the Gnuplotting-Stuff for two other classes"""
    def __init__(self,fname,smallestFreq=0.,persist=None,splitThres=2048,plotLinear=True,plotCont=True,plotBound=True):
        StepAnalyzedCommon.__init__(self,fname,BoundingPlotLogAnalyzer(),smallestFreq=smallestFreq)

        self.plot=None
        self.plotCont=None
        self.plotBound=None

        if plotLinear:
            self.plot=GnuplotTimelines(self.getAnalyzer("PlotLinear").lines,persist=persist)
            self.getAnalyzer("PlotLinear").lines.setSplitting(splitThres=splitThres,splitFun=max)
        
            self.plot.set_string("logscale y")
            self.plot.title("Residuals")

        if plotCont:
            self.plotCont=GnuplotTimelines(self.getAnalyzer("PlotContinuity").lines,persist=persist)
            self.getAnalyzer("PlotContinuity").lines.setSplitting(splitThres=splitThres)

            self.plotCont.title("Continuity")
 
        if plotBound:
            self.plotBound=GnuplotTimelines(self.getAnalyzer("PlotBounding").lines,persist=persist)
            self.getAnalyzer("PlotBounding").lines.setSplitting(splitThres=splitThres,splitFun=signedMax)
            self.plotBound.title("Bounded variables")

    def timeHandle(self):
        if self.plot:
            self.plot.redo()
        if self.plotCont:
            self.plotCont.redo()
        if self.plotBound:
            self.plotBound.redo()

    def stopHandle(self):
        self.timeHandle()
        
        
class GnuplotRunner(GnuplotCommon,BasicRunner):
    def __init__(self,argv=None,smallestFreq=0.,persist=None,plotLinear=True,plotCont=True,plotBound=True):
        """@param smallestFreq: smallest Frequency of output
        @param persist: Gnuplot window persistst after run"""
        BasicRunner.__init__(self,argv=argv)
        GnuplotCommon.__init__(self,"Gnuplotting",smallestFreq=smallestFreq,persist=persist,plotLinear=plotLinear,plotCont=plotCont,plotBound=plotBound)

class GnuplotWatcher(GnuplotCommon,BasicWatcher):
    def __init__(self,logfile,smallestFreq=0.,persist=None,silent=False,tailLength=1000,sleep=0.1,plotLinear=True,plotCont=True,plotBound=True):
        """@param smallestFreq: smallest Frequency of output
        @param persist: Gnuplot window persistst after run"""
        BasicWatcher.__init__(self,logfile,silent=silent,tailLength=tailLength,sleep=sleep)
        GnuplotCommon.__init__(self,logfile,smallestFreq=smallestFreq,persist=persist,plotLinear=plotLinear,plotCont=plotCont,plotBound=plotBound)

    def startHandle(self):
        self.bakFreq=self.freq
        self.freq=3600
        
    def tailingHandle(self):
        self.freq=self.bakFreq
        self.oldtime=0
        
        
